#!/usr/local/bin/perl -w

use strict;
# use LWP::Simple;
use vars qw( %task $me );

$task{$me}{timespec} = '2 * * * *';
$task{$me}{timespec_panic_1} = '';
$task{$me}{code} = sub {
# This is GPL'd code from portald:
################################################################################
    sub geturl {
	use LWP::UserAgent;
	use HTTP::Request;
	my $ua = new LWP::UserAgent;
	my $request = new HTTP::Request('GET', $_[0]);
	$ua->timeout(30);
	my $result = $ua->request($request);

	if ($result->is_success) {
	    return $result->content;
	} else {
	    return 0;
	}
    }

    sub _dcdate2timestamp {
# dc:time 2003-06-20T14:34+08:00
# or 2003-06-28T01:22:24+08:00
	use Date::Parse;
	my $d = shift;
	$d =~ s/:(\d\d)$/$1/;
	my $t = str2time($d);
	return $t;
    }

    sub _timestamp2storytime {
# format 2003-06-23 22:18:03
	use Date::Format;
	my $t = shift;
	# Story cover date is ALWAYS saved in GMT, to let user
	# timezone displayed correctly.
	return time2str("%Y-%m-%d %T",$t,"GMT");
    }

    sub _timestamp2sid {
# Code copied from createStory()
# yes, this format is correct, don't change it :-)
	my $sidformat = '%02d/%02d/%02d/%02d%0d2%02d';
# Create a sid based on the current time.
	my $start_time = shift;
	my @lt = localtime($start_time);
	$lt[5] %= 100; $lt[4]++; # year and month
	    my $sid = sprintf($sidformat, @lt[reverse 0..5]);
	return $sid;
    }

################################################################################

    slashdLog("RSS-Import.pl begins");
    my($virtual_user, $constants, $slashdb, $user) = @_;
    my $rsslist = $slashdb->sqlSelectAllHashrefArray(
	    "uid,tid,rssurl", "rss2storyrssurls", "", "");
# CHANGE THIS to a directory writable by Slashd
    my $rssfiledir = "/tmp/rss2story";
    mkdir($rssfiledir) unless -d $rssfiledir;

    my $uid = $slashdb->sqlSelect('uid','users',"nickname='rss-robot'");

# for each RDF
    my $totalrdfbytes = 0;
    foreach (@$rsslist) {
	my $url = $_->{rssurl};
	my $tid = $_->{tid};
	slashdLog("*** $url ***");
# Get it
	my $content = geturl($url);
	$totalrdfbytes += length($content) + 100; # header overhead
	    next if !defined($content) || (length($content) < 50);
# Change / to # for a valid filename
	my $filename = $url;
	$filename =~ s/\//#/g;
	$filename = "$rssfiledir/$filename";
	my $oldfile = "$filename.old";
	rename $filename, $oldfile if -f $filename;  # keep the old!
# Save it to a file
	    open (F, ">$filename");
	print F $content;
	close F;
# Parse it into an XML::RDF tree
	use XML::RSS;
	my $rss = new XML::RSS;
	$rss->add_module(prefix => "content", uri => "http://web.resource.org/rss/1.0/modules/content/");
	eval { $rss->parse($content); };
	if ($@) {  # if it errored out
	    slashdLog("Error processing XML");
	    next;
	}
	my $oldrss = new XML::RSS;
	my $old = -f $oldfile;
	eval { $oldrss->parsefile($oldfile) if $old; };
	if ($@) {
	    slashdLog("Error processing old XML");
	    next;
	}

	my @newitems = @{$rss->{'items'}};
	my @olditems;
	@olditems = @{$oldrss->{'items'}} if $old;
	my @items;
	my $rsscreator = $rss->{dc}->{creator};
	foreach my $item (@newitems) {
	    if (grep(/$item->{'link'}/, map {$_->{'links'}} @olditems  )) {
	    	# this entry was in the old rdf. so it should be imported already.
	    	next;
	    }
	    my $dc = $item->{dc};
	    my $dccreator = $dc->{creator} || $rsscreator;
# check necessary tags
	    unless( $item->{title} &&
		    $dc->{date}    &&
		    $dccreator     &&
		    $item->{link}  &&
		    $item->{description}) {
		slashdLog("Item Ignored: Missing tags (title/dc:date/dc:creator/link/descripition");
		next;
	    }
	    push @items, $item;
	}

	my $nitems = scalar(@items);
	slashdLog("This url has $nitems itmes");

# Find the new stuff
	foreach my $item (@items) {
	    my $dc = $item->{dc};

# XXX: Check to see if we already had this story.
	    my $qURL = $slashdb->sqlQuote($item->{link});
	    my $urisid = $slashdb->sqlSelect('sid','rss2story_fulltext',"uri=$qURL");
	    if($urisid) {
	    	slashdLog("=> Story already exists, ignored.($urisid,$qURL)");
		next;
	    }
	    my $newsid = _timestamp2sid _dcdate2timestamp($dc->{date});
	    my $qnewsid = $slashdb->sqlQuote($newsid);
	    my $oldsid = $slashdb->sqlSelect('sid','stories',"sid=$qnewsid");
	    if($newsid == $oldsid) {
	    	slashdLog("=> Story already exists, ignored.($newsid) (but have no record in rss2story_fulltext!). Should be a cover-date confliction");
		next;
	    }
# XXX

# Now we know we're at a new item
# so post it as a story!!!
	    my $storyform;
	    $storyform->{submitter} = $uid;
	    $storyform->{uid} = $uid;
	    $storyform->{title} = $item->{title};
	    $storyform->{displaystatus} = 0;
	    $storyform->{commentstatus} = 0;

# Reformat dc:time to sql format.
	    my $storytime = _timestamp2storytime
		_dcdate2timestamp($dc->{date});
	    $storyform->{'time'} = $storytime ;

	    $storyform->{introtext} = $dc->{creator} . qq# wrote : "$item->{'description'} [<a href="$item->{'link'}">More...</a>]"#;
	    $storyform->{bodytext} = "";
	    $storyform->{relatedtext} = "";
	    $storyform->{tid} = $tid;
	    $storyform->{section} = 'rss2story';
	    $storyform->{dept} = $dc->{subject} || "rss-aggregator";

	    my $sid = $slashdb->createStory($storyform);
	    if ($sid) {
# Change sid. Evil
# XXX - Probablly would causing dup sid.
		my $newsid = _timestamp2sid
		    _dcdate2timestamp($dc->{date});
		my $h = {sid => $newsid};
		$slashdb->sqlUpdate("stories",$h,"sid='${sid}'");
		$slashdb->sqlUpdate("story_param",$h,"sid='${sid}'");
		$slashdb->sqlUpdate("story_text",$h,"sid='${sid}'");
		$sid = $newsid;

		slashdLog("$0: story created: " . $item->{'link'});
		my $id = $slashdb->createDiscussion( {
			title => $storyform->{title},
			section => $storyform->{section},
			topic => $storyform->{tid},
			url => "$constants->{rootdir}/article.pl?sid=$sid",
			sid => $sid,
			ts => $storyform->{time}
			} );
# XXX Insert it's fulltext content in to a table sothat it is searchable.
		my $fulltxt = $item->{content}->{encoded};
		$slashdb->sqlInsert('rss2story_fulltext',
			{
			sid     => $sid,
			uri     => $item->{'link'},
			fulltxt => $fulltxt,
			});

		if ($id) {
		    $slashdb->setStory($sid, { discussion => $id });
		} else {
		    slashdLog("Can't create discussion for story $sid, title $storyform->{title}");
		}
	    }else {
		slashdLog("Somehow story creation is failed for " . $item->{'link'});
	    }
	}
    }

    slashdLog("RDF Bytes Transmitted: $totalrdfbytes");
    slashdLog("RSS-import.pl ends");
    return;
};

1;

