#!/usr/local/bin/perl -w

use strict;
use Slash 2.001;	# require Slash 2.1
use Slash::Display;
use Slash::Utility;
use Slash::XML;
use vars qw($VERSION);

($VERSION) = '0.01';

use constant ALLOWED    => 0;
use constant FUNCTION   => 1;

# my 'rss2storyrssurls' = 'rss2storyrssurls';

sub main {
#	my $self      = getObject('Slash::RSS2Story');
	my $constants = getCurrentStatic();
	my $slashdb   = getCurrentDB();
	my $user      = getCurrentUser();
	my $form      = getCurrentForm();

	# possible value of "op" parameter in form
 	my %ops = (
		searchArticle  => [ 1, \&searchArticle ],
		searchFullTxt  => [ 1, \&searchArticleFullText ],
 		addURL	=> [ !$user->{is_anon},	\&addURL ] ,
 		delURL	=> [ !$user->{is_anon},	\&delURL ] ,
 		default	=> [ !$user->{is_anon},	\&showMain ],
 	);

 	my $op = $form->{'op'};
 	if (!$op || !exists $ops{$op} || !$ops{$op}[ALLOWED]) {
 		$op = 'default';
 	}

 	# hijack RSS feeds
 	if ($form->{content_type} eq 'rss') {
 	    # maybe just show RSS feed, aka html2rss, here?
 	} else {
 	    $ops{$op}[FUNCTION]->($constants, $user, $form, $slashdb);
	}
}

sub _getRSSList {
	my ($slashdb,$user) = @_;
	my $uid	= $user->{uid};
	return $slashdb->sqlSelectAllHashrefArray("*", 'rss2storyrssurls' ,"uid=${uid}");
}

sub _insertRSSURL {
	my ($user,$slashdb,$newrssurl,$newdesc) = @_;
	return unless $slashdb->sqlInsert('rss2storyrssurls', {
		uid	=> $user->{uid},
		rssurl	=> $newrssurl,
		description => $newdesc
	});
	return $slashdb->getLastInsertId('rss2storyrssurls','id');
}

sub delURL {
	my($constants, $user, $form, $slashdb) = @_;

	my @del;
	foreach (keys %$form) {
	    push @del,$1 if(/^delurl_(\d+)/) ;
	}

	my $uid   = $user->{uid};
	my $title = "Nothing is deleted.";
	if(length(@del)) {
	    foreach (@del) {
		my $tid   = $slashdb->sqlSelect("tid","rss2storyrssurls","id=$_");
		$slashdb->sqlDo("DELETE from rss2storyrssurls where id=$_ and uid=$uid");
		$slashdb->sqlDo("DELETE from topics where tid=$tid");
	    }
	    $title =  $#del +1 . " RSS-URL(s) deleted.";
	}

	my $rsslist   = _getRSSList($slashdb,$user);

	header("$constants->{sitename}: RSS2Stroy / Delete URL");
	slashDisplay('main', {
		title	=> $title,
		user	=> $user,
		rsslist => $rsslist
	});
	footer();

}

sub addURL {
	my( $constants, $user, $form, $slashdb) = @_;

	my $newrssurl = $form->{newrssurl};
	my $newdesc   = $form->{newdesc};

	my $id = _insertRSSURL($user,$slashdb,$newrssurl,$newdesc); 
	my $title = $id ? "New RSS Added":"Failed to add new RSS URL. This url already exists.";

	if($id) {
	    # Create a new topic for this RSS.
	    my $f = {
		tid	=> '',
		op	=> 'topics',
		name	=> $newdesc,
		alttext	=> $newdesc,
		image	=> 'topicrss.jpg',
		topicnew => 'Create new topic' ,
	    };
	    $f->{tid} = $slashdb->saveTopic($f);
	    $slashdb->sqlUpdate('rss2storyrssurls',{ tid => $f->{tid} },"id=$id");
	    # Associate this topic to 'rss2story' section
	    $slashdb->sqlInsert('section_topics',{
			    section => 'rss2story',
			    tid     => $f->{tid},
			    });

	}

	my $rsslist   = _getRSSList($slashdb,$user);
	header("$constants->{sitename}: RSS2Stroy / Add URL");
	slashDisplay('main', {
		title	=> $title,
		user	=> $user,
		rsslist => $rsslist
	});
	footer();
}

sub showMain {
	my( $constants, $user, $form, $slashdb) = @_;
	my $rsslist   = _getRSSList($slashdb,$user);

	if($user->{is_anon}) { return showAnonMain(@_); }

	header("$constants->{sitename}: RSS2Stroy");
	slashDisplay('main', {
		user	=> $user,
		rsslist => $rsslist
	});
	footer();
}

sub showAnonMain {
	my( $constants, $user, $form, $slashdb) = @_;

	header("$constants->{sitename}: RSS2Stroy");
	print "<h1>Anonymous user are not allowed to enter this page</h1>";
	footer();
}

sub searchArticle {
	my( $constants, $user, $form, $slashdb) = @_;
	my $title = $form->{title};
	my $qtitle = qq{"$title"};
 	my $r = $slashdb->sqlSelectHashref('*','stories',"title=$qtitle");
	my $sid=$r->{sid};
	my $url = $constants->{rootdir} . "/article.pl?sid=${sid}&mode=thread";
	print qq{<html><head>
 <meta http-equiv="REFRESH" content="0; URL=$url">
</head><body></body></html>};

}

sub searchArticleFullText {
	my( $constants, $user, $form, $slashdb) = @_;
	my $words = $form->{words};

	unless ($words) {
		header("$constants->{sitename}: Full-Text Search");
		slashDisplay('searchForm');
		footer();
		return 1;
	}

	if($user->{seclev} < 500) {
		header("$constants->{sitename}: Full-Text Search");
		slashDisplay('permissionDenied');
		footer();
		return 0;
	}

	my $qwords = '"%' . ${words} . '%"';
# 	my $r = $slashdb->sqlSelectHashref('*','rss2story_fulltext',"fulltxt like $qwords");
	my $r = $slashdb->sqlSelectAllHashrefArray('*','rss2story_fulltext,stories',"fulltxt like $qwords and stories.sid = rss2story_fulltext.sid ");
	if( @$r ) {
#		use Data::Dumper;
		header("$constants->{sitename}: Search Result");
		slashDisplay('searchResult', {
			stories => $r
		});
#		foreach (@$r) {
#			my $qsid = $_->{sid}; $qsid = qq{"$qsid"};
#			my $s = $slashdb->sqlSelectHashref('*','stories',"sid=$qsid");
#			print "<pre>" . Dumper($_). "</pre>\n";
#		}
		footer();
	} else {
		header("$constants->{sitename}: Search");
		print "<h1>NOT FOUND</h1>";
		footer();
	}
}

createEnvironment();
main();
1;
