#!/usr/local/bin/perl -w

# SlashWiki program for Slash 2.2 or greater
# this file is available under the GNU General Public Licence
# it is also copyright chromatic 2002
# see the included GPL file for details

use strict;

use Slash;
use Slash::Display;
use Slash::Utility;
use Text::WikiFormat;

use vars qw ( %ops $VERSION %Elix %utf8_cache );

$VERSION = '0.05';

%ops = (
	update	=> \&update,
	list	=> \&list,
	delete	=> \&delete,
	display => \&display,
);

dbmopen(%Elix, '/tmp/elix', 0444) or die $! if -r '/tmp/elix';

use Encode::HanConvert qw/simple trad/;
$0 =~ /(simp|trad)/;
our $func = ($1 eq 'trad') ? 'Encode::HanConvert::trad' :
	    ($1 eq 'simp') ? 'Encode::HanConvert::simple' 
	    : undef;

main();
return();

sub tran {
    no strict 'refs';
    return unless $func;
    warn $0;

    foreach my $item (@_) {
	$item = join (
	    '', map {
		$func->($_)
	    } split (
		/\n/, $item
	    )
	)
    }
}

sub main {
	# local $Slash::Display::Transform = \&to_utf8;

	use URI::Escape;
	my $form = getCurrentForm();

	my $op = $form->{op};
	unless (exists $ops{$op}) {
		$op = 'display';
	}
	
	$ops{$op}->($form);
}

sub display {
	my $form	= shift;
	my $slashdb	= getCurrentDB();
	my $constants	= getCurrentStatic();

	# redirect unless $user->{seclev} >= $constants->{wikiWrite}

	my $page	= $form->{page} || $constants->{wikiDefaultPage} || 'default';
	header(nice_utf8("$page"));

	my $version = $form->{version} || '';
	$version	= 0 if $version =~ /\D/;

	unless ($slashdb->sqlSelect('wid', 'wikipage', 'title = '.$slashdb->sqlQuote($page))) {
	    my $new_page = to_utf8($page);
	    if ($slashdb->sqlSelect('wid', 'wikipage', 'title = '.$slashdb->sqlQuote($new_page))) {
		$page = $new_page;
	    }
	}

	my $where = "title = " . $slashdb->sqlQuote($page) . 
		" AND wikipage.wid = wikitext.wid ";

	
	if ($version) {
		$where .= "AND wikitext.version = " . $slashdb->sqlQuote($version);
	} else {
		$where .= "AND wikipage.version = wikitext.version ";
	}

	my %page_data = (
		title	=> $page,
	);

	$where .= ' AND users.uid = wikitext.uid';

	@page_data{qw(raw cooked nickname date wid version curver)} = 
	$slashdb->sqlSelect('raw, cooked, nickname, date, wikipage.wid,' .
		'wikitext.version, wikipage.version',
		'wikipage, wikitext, users', $where
	);

	$page_data{readonly} = ($version && $version == $page_data{curver});
	$page_data{date} = timeCalc($page_data{date});

	$page_data{elix} = to_utf8($Elix{lc($page)});

	tran(@page_data{qw/title elix cooked/});

	slashDisplay('wikiView', \%page_data);
	footer();
}

sub list {
	my $form		= shift;
	my $slashdb		= getCurrentDB();
	my $constants	= getCurrentStatic();

	my $page = $form->{page} || '';

	my ($pagelist, $template);
	if ($page) {
		my $revs = $form->{revisionLimit} || $constants->{wikiRevisionLimit};
		$revs = 25 unless $revs and $revs !~ /\D/;

		header(nice_utf8("Latest $revs Revisions of $page"));
		$pagelist = [ map { 
			$_->{date} = timeCalc($_->{date});
			$_;
		} @{ $slashdb->sqlSelectAllHashrefArray(
			'wikitext.version, title, description, date, nickname',
			'wikipage, wikitext, users', 
			'wikipage.wid = wikitext.wid AND users.uid = wikitext.uid ' .
			"AND title = " . $slashdb->sqlQuote($page), 
			"order by version DESC limit $revs"
		) }];
		$template = 'wikiRevisions';
	} else {
		my $recents = $form->{recentLimit} || $constants->{wikiRecentLimit};
		$recents = 25 unless $recents and $recents !~ /\D/;

		header("Latest $recents Modified Pages");

		$pagelist = [ map { 
			$_->{date} = timeCalc($_->{date});
			$_;
		} @{ $slashdb->sqlSelectAllHashrefArray(
			'wikipage.version, title, date, description, nickname', 
			'wikipage, wikitext, users', 
			'wikipage.wid = wikitext.wid AND users.uid = wikitext.uid ' .
			'AND wikipage.version = wikitext.version', 
			"order by date DESC limit $recents"
		)} ];
		$template = 'wikiRecent';
	}

	slashDisplay($template, {
		page		=> $page,
		pagelist	=> $pagelist,
	});

	footer();
}

sub update {
	my $form		= shift;
	my $user		= getCurrentUser();
	my $slashdb		= getCurrentDB();
	my $constants	= getCurrentStatic();

	my ($raw, $description, $title, $wid) = 
		@$form{qw( raw description title wid )};
	my ($version, $date);

	# verify data
	return if ($wid =~ /\D/ or $version =~ /\D/);
	$raw =~ tr/\r//d;

	# redirect unless $user->{seclev} >= $constants->{wikiWrite}
	my $cooked = Text::WikiFormat::format($raw, {}, { 
		prefix		=> "$constants->{rootdir}/wiki.pl?page=",
		extended	=> $constants->{wikiExtendedLinks},
		implicit	=> ($constants->{wikiImplicitLinks} ne '0'),
	});

	unless ($form->{preview}) {
		my ($method, $where);
		if ($wid) {
			$where = "wid = $wid AND title = " . $slashdb->sqlQuote($title);
			$method	= 'sqlUpdate';
			$version = $slashdb->sqlSelect('version', 'wikipage', $where) + 1; 
		} else {
			$wid = $slashdb->sqlSelect('MAX(wid)+1','wikipage');
			$where = '';
			$method	= 'sqlInsert';
		}

		$version ||= 1;

		my $rows = $slashdb->$method('wikipage', {
			title		=> $form->{title},
			version		=> $version,
		}, $where) or warn "Could not update database\n";

		$slashdb->sqlInsert('wikitext', {
			raw			=> $raw,
			wid			=> $wid,
			uid			=> $user->{uid},
			-date		=> 'now()',
			cooked		=> $cooked,
			version		=> $version,
			description	=> $description,
		}) or warn "Could not update database\n";
	}

	header(nice_utf8("$title"));

	slashDisplay('wikiView', {
		raw			=> $raw,
		wid			=> $wid,
		date		=> timeCalc($slashdb->getTime()),
		title		=> $title,
		cooked		=> $cooked,
		nickname	=> $user->{nickname},
		description	=> $description,
		elix		=> to_utf8($Elix{lc($title)}),
	});
	footer();
}

sub delete {}

sub to_utf8 {
    return ($utf8_cache{$_[0]} ||= do {
	my $f = "/tmp/big5.".rand();
	open _, ">$f" or die $!;
	print _ $_[0];
	close _;
	chomp(my $result = `/usr/local/bin/iconv -f big5 -t utf-8 $f`);
	unlink $f;
	$result;
    });
}

sub nice_utf8 {
    join("", map(($_ > 255 ?  "&#$_;" : chr), unpack("U0U*", $_[0])));
}

1;
